@@ -5,7 +5,7 @@ poolMAlloc <- function(nPerPar, facPlc, nAllocStart,
    dataset, stopProp, stopValue,
    selectParam = NULL,
    double = FALSE, checkConv=FALSE, 
-   names='default', leaveout=0,useTotalAlloc=FALSE, ...) 
+   names='default', leaveout=0, useTotalAlloc=FALSE, ...) 
 {
  
   StartTimeFull <- proc.time()
@@ -17,55 +17,6 @@ poolMAlloc <- function(nPerPar, facPlc, nAllocStart,
   
   
   
-## isImproperSolution function
-
-isProperSolution <- function(object) {
-
-    lavpartable <- object@ParTable
-    lavfit <- object@Fit
-    lavdata <- object@Data
-    lavmodel <- object@Model
-  
-
-       # 1. check for Heywood cases, negative (residual) variances, ...
-        var.idx <- which(lavpartable$op == "~~" &
-                         lavpartable$lhs == lavpartable$rhs)
-        if(length(var.idx) > 0L && any(lavfit@est[var.idx] < 0.0))
-            return(FALSE)
-
-        # 2. is cov.lv (PSI) positive definite?
-        if(length(lavaan::lavaanNames(lavpartable, type="lv.regular")) > 0L) {
-            ETA <- list(lavaan::lavInspect(object,"cov.lv"))
-            for(g in 1:lavdata@ngroups) {
-                eigvals <- eigen(ETA[[g]], symmetric=TRUE,
-                                 only.values=TRUE)$values
-                if(any(eigvals < -1 * .Machine$double.eps^(3/4)))
-                   return(FALSE)
-            }
-            
-            }
-        
-
-        # 3. is THETA positive definite (but only for numeric variables)
-        THETA <- list(lavaan::lavInspect(object,"theta"))
-        
-        for(g in 1:lavdata@ngroups) {
-                num.idx <- lavmodel@num.idx[[g]]
-                if(length(num.idx) > 0L) {
-                    eigvals <- eigen(THETA[[g]][unlist(num.idx),unlist(num.idx),drop=FALSE],
-                                     symmetric=TRUE,
-                                     only.values=TRUE)$values
-                    if(any(eigvals < -1 * .Machine$double.eps^(3/4))) {
-                       return(FALSE)
-                     }
-                }
-        }
- 
-        
-   TRUE
-
-}  
-
   {
   nloop <- 0
   ### start loop counter
@@ -388,13 +339,13 @@ if(Maxv > 0){
     ## convert allocation matrix to dataframe for model estimation 
     fit <- lavaan::sem(syntax, control=list(iter.max=100), data=data_parcel, ...)
     ## estimate model in lavaan
-	if (inspect(fit, "converged")==TRUE){
+	if (lavaan::lavInspect(fit, "converged")==TRUE){
 	Converged[[i]] <- 1
 	} else Converged[[i]] <- 0
 	## determine whether or not each allocation converged
     Param[[i]] <- lavaan::parameterEstimates(fit)[,c("lhs","op","rhs","est","se","z","pvalue","ci.lower","ci.upper")]
     ## assign allocation parameter estimates to list
-    if (isProperSolution(fit)==TRUE & Converged[[i]]==1) {
+    if (lavaan::lavInspect(fit, "post.check") & Converged[[i]] == 1) {
 	ProperSolution[[i]] <- 1
 	} else ProperSolution[[i]] <- 0
 	## determine whether or not each allocation has proper solutions
